"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toJsonSchema = exports.fromJsonSchema = exports.Recursive = void 0;
var _registry = /*#__PURE__*/require("@traversable/registry");
var _schema = /*#__PURE__*/require("@traversable/schema");
var _properties = /*#__PURE__*/require("./properties.js");
var JsonSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./jsonSchema.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const phantom = () => void 0;
/** @internal */
function handleOptionality(x) {
  let {
    [_registry.symbol.optional]: ix,
    ...y
  } = x;
  y = Recursive.fromJsonSchema(y);
  if (typeof ix !== 'number') return y;else while (ix-- > 0) y = _schema.t.optional(y);
  return y;
}
function handleNullability(x) {
  let {
    nullable,
    ...y
  } = x;
  return _schema.t.optional(Recursive.fromJsonSchema(y));
}
/** @internal */
function recoverTupleOptionality(xs, {
  min,
  max
}) {
  if (min !== max) {
    const firstOptional = min;
    const req = xs.slice(0, firstOptional);
    const opt = xs.slice(firstOptional).map(_schema.t.optional.def);
    return [...req, ...opt];
  } else return xs;
}
var Recursive;
(function (Recursive) {
  Recursive.toJsonSchema = x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case x.tag === _registry.URI.symbol:
        return phantom;
      case x.tag === _registry.URI.bigint:
        return phantom;
      case x.tag === _registry.URI.never:
        return phantom;
      case x.tag === _registry.URI.undefined:
        return phantom;
      case x.tag === _registry.URI.void:
        return phantom;
      case x.tag === _registry.URI.any:
        return () => JsonSchema.RAW.any;
      case x.tag === _registry.URI.unknown:
        return () => JsonSchema.RAW.any;
      case x.tag === _registry.URI.null:
        return () => JsonSchema.RAW.null;
      case x.tag === _registry.URI.boolean:
        return JsonSchema.boolean;
      case x.tag === _registry.URI.integer:
        return () => JsonSchema.integer(x);
      case x.tag === _registry.URI.number:
        return () => JsonSchema.number(x);
      case x.tag === _registry.URI.string:
        return () => JsonSchema.string(x);
      case x.tag === _registry.URI.optional:
        return JsonSchema.optional(x.def()).toJsonSchema;
      case x.tag === _registry.URI.eq:
        return () => JsonSchema.eq(x.def);
      case x.tag === _registry.URI.array:
        return () => JsonSchema.array(x.def(), {
          minLength: x.minLength,
          maxLength: x.maxLength
        });
      case x.tag === _registry.URI.record:
        return () => JsonSchema.record(x.def());
      case x.tag === _registry.URI.union:
        return () => JsonSchema.union(_registry.fn.map(x.def, v => v()));
      case x.tag === _registry.URI.intersect:
        return () => JsonSchema.intersect(_registry.fn.map(x.def, v => v()));
      case x.tag === _registry.URI.tuple:
        return () => JsonSchema.tuple(_registry.fn.map(x.def, v => v()));
      case x.tag === _registry.URI.object:
        {
          const required = Object.keys(x.def).filter((0, _properties.isRequired)(x.def));
          return () => ({
            type: 'object',
            required,
            properties: _registry.fn.map(x.def, (v, k) => (0, _properties.property)(required)(v(), k))
          });
        }
    }
  };
  const fromJsonSchemaInteger = ({
    minimum: min,
    maximum: max,
    exclusiveMinimum: xMin,
    exclusiveMaximum: xMax
  } = {
    type: 'integer'
  }) => {
    let out = _schema.t.integer;
    let minimum = _schema.t.number(min) && !_schema.t.boolean(xMin) ? min : void 0;
    let maximum = _schema.t.number(max) && !_schema.t.boolean(xMax) ? max : void 0;
    if (_schema.t.number(minimum)) out = out.min(minimum);
    if (_schema.t.number(maximum)) out = out.max(maximum);
    return out;
  };
  const fromJsonSchemaNumber = ({
    minimum: min,
    maximum: max,
    exclusiveMinimum: xMin,
    exclusiveMaximum: xMax
  } = {
    type: 'number'
  }) => {
    let out = _schema.t.number;
    let exclusiveMinimum = _schema.t.number(xMin) ? xMin : _schema.t.boolean(xMin) && _schema.t.number(min) ? min : void 0;
    let exclusiveMaximum = _schema.t.number(xMax) ? xMax : _schema.t.boolean(xMax) && _schema.t.number(max) ? max : void 0;
    let minimum = _schema.t.number(min) && !_schema.t.boolean(xMin) ? min : void 0;
    let maximum = _schema.t.number(max) && !_schema.t.boolean(xMax) ? max : void 0;
    if (_schema.t.number(exclusiveMinimum)) out = out.moreThan(exclusiveMinimum);
    if (_schema.t.number(exclusiveMaximum)) out = out.lessThan(exclusiveMaximum);
    if (_schema.t.number(minimum)) out = out.min(minimum);
    if (_schema.t.number(maximum)) out = out.max(maximum);
    return out;
  };
  const fromJsonSchemaString = ({
    minLength: min,
    maxLength: max
  } = {
    type: 'string'
  }) => {
    let out = _schema.t.string;
    if (_schema.t.number(min)) out = out.min(min);
    if (_schema.t.number(max)) out = out.max(max);
    return out;
  };
  Recursive.fromJsonSchema = x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case JsonSchema.is.any(x):
        return _schema.t.unknown;
      case _registry.symbol.optional in x:
        return handleOptionality(x);
      case 'nullable' in x:
        return handleNullability(x);
      case JsonSchema.is.optional(x):
        return _schema.t.optional.def(x);
      case JsonSchema.is.null(x):
        return _schema.t.null;
      case JsonSchema.is.boolean(x):
        return _schema.t.boolean;
      case JsonSchema.is.integer(x):
        return fromJsonSchemaInteger(x);
      case JsonSchema.is.number(x):
        return fromJsonSchemaNumber(x);
      case JsonSchema.is.string(x):
        return fromJsonSchemaString(x);
      case JsonSchema.is.record(x):
        return _schema.t.record(x.additionalProperties);
      case JsonSchema.is.union(x):
        return _schema.t.union.def(x.anyOf);
      case JsonSchema.is.enum(x):
        return _schema.t.union.def(x.enum.map(_ => _schema.t.eq(_)));
      case JsonSchema.is.const(x):
        return _schema.t.eq.def(x.const);
      case JsonSchema.is.intersect(x):
        return _schema.t.intersect.def(x.allOf);
      case JsonSchema.is.tuple(x):
        return _schema.t.tuple.def(recoverTupleOptionality(x.items, {
          min: x.minItems,
          max: x.maxItems
        }));
      case JsonSchema.is.array(x):
        return _schema.t.array.def(x.items, x);
      case JsonSchema.is.object(x):
        return _schema.t.object(_registry.fn.map(x.properties, (v, k) => x.required.includes(`${k}`) ? v : _schema.t.optional(v)));
    }
  };
})(Recursive || (exports.Recursive = Recursive = {}));
/**
 * ## {@link fromJsonSchema `fromJsonSchema`}
 *
 * Recursively convert a JSON schema into a traversable schema.
 *
 * __Roundtrip property:__
 *
 * Thoroughly tested using randomly generated schemas to be invertible when paired
 * with {@link toJsonSchema} without any loss of information.
 */
const fromJsonSchema = exports.fromJsonSchema = /*#__PURE__*/JsonSchema.fold(Recursive.fromJsonSchema);
/**
 * ## {@link toJsonSchema `toJsonSchema`}
 *
 * Recursively convert a traversable schema into its JSON schema representation.
 *
 * __Roundtrip property:__
 *
 * Thoroughly tested using randomly generated schemas to be invertible with
 * {@link fromJsonSchema `fromJsonSchema`} without any loss of information.
 *
 * __Note:__ If you find yourself reaching for {@link toJsonSchema `toJsonSchema`},
 * keep in mind that by installing the `@traversable/schema-to-json-schema` package,
 * all of the built-in schemas from the `@traversable/schema` package come
 * equipped with a `.toJsonSchema` getter method.
 *
 * Prefer using the `.toJsonSchema` method if possible -- not only is it
 * easier to use, but it's also more performant, since the implementation
 * is not recursive.
 */
const toJsonSchema = exports.toJsonSchema = /*#__PURE__*/_schema.t.fold(Recursive.toJsonSchema);
//# sourceMappingURL=recursive.js.map