import type { Force, PickIfDefined, Returns } from '@traversable/registry';
import { symbol as Sym } from '@traversable/registry';
import type { MinItems } from './items.js';
import type { RequiredKeys } from './properties.js';
import * as Spec from './specification.js';
import { t } from '@traversable/schema';
export type { Unary, Free, Nullary, NumericBounds, } from './specification.js';
export { is, RAW, } from './specification.js';
export { Functor, fold, unfold, } from './functor.js';
export type { RequiredKeys } from './properties.js';
export { isRequired, property, wrapOptional, } from './properties.js';
export type { MinItems } from './items.js';
export { minItems } from './items.js';
export type { JsonSchema, LowerBound, Schema, };
type Nullable<T> = Force<T & {
    nullable: true;
}>;
type JsonSchema<T = never> = [T] extends [never] ? Spec.JsonSchema : Spec.Unary<T>;
type StringBounds<T> = Force<{
    type: 'string';
} & PickIfDefined<T, keyof Spec.SizeBounds>>;
type NumberBounds<T> = Force<{
    type: 'number';
} & PickIfDefined<T, keyof Spec.NumericBounds>>;
type IntegerBounds<T> = Force<{
    type: 'integer';
} & PickIfDefined<T, keyof Spec.NumericBounds>>;
type ArrayBounds<S, T> = Force<{
    type: 'array';
    items: Returns<S['toJsonSchema' & keyof S]>;
} & PickIfDefined<T, keyof Spec.SizeBounds>>;
interface LowerBound {
    toJsonSchema(): JsonSchema | undefined;
}
interface Schema {
    toJsonSchema?(): any;
}
export { Empty as never, Empty as void, Empty as symbol, Empty as undefined, Empty as bigint, };
export declare function empty(): never;
declare const Empty: {
    toJsonSchema: typeof empty;
};
interface Empty {
    toJsonSchema(): undefined;
}
export interface inline<S> {
    toJsonSchema: () => void;
}
export declare function inline<S>(_: S): inline<S>;
export type { any_ as any };
interface any_ {
    toJsonSchema(): typeof Spec.RAW.any;
}
export type { unknown_ as unknown };
interface unknown_ {
    toJsonSchema(): typeof Spec.RAW.any;
}
export type { null_ as null };
interface null_ {
    toJsonSchema(): typeof Spec.RAW.null;
}
export { boolean_ as boolean };
interface boolean_ {
    toJsonSchema(): typeof Spec.RAW.boolean;
}
declare const boolean_: () => {
    type: "boolean";
};
export interface integer {
    toJsonSchema(): IntegerBounds<this>;
}
export declare function integer(schema: t.integer): {
    minimum?: number | undefined;
    maximum?: number | undefined;
    exclusiveMinimum?: number | boolean | undefined;
    exclusiveMaximum?: number | boolean | undefined;
    type: "integer";
};
export { number_ as number };
interface number_ {
    toJsonSchema(): NumberBounds<this>;
}
declare function number_(schema: t.number): {
    minimum?: number | undefined;
    maximum?: number | undefined;
    exclusiveMinimum?: number | boolean | undefined;
    exclusiveMaximum?: number | boolean | undefined;
    type: "number";
};
export { string_ as string };
interface string_ {
    toJsonSchema(): StringBounds<this>;
}
declare function string_(schema: t.string): never;
export interface eq<S> {
    toJsonSchema(): {
        const: S;
    };
}
export declare function eq<V>(value: V): {
    const: V;
};
export interface optional<S> {
    toJsonSchema: {
        [Sym.optional]: number;
        (): Nullable<Returns<S['toJsonSchema' & keyof S]>>;
    };
}
export declare function optional<T>(x: T): optional<T>;
export declare function optionalProto<T>(child: T): any;
export interface array<S> {
    toJsonSchema(): ArrayBounds<S, this>;
}
export declare function array<T, Min extends number, Max extends number>(itemsType: T, bounds?: {
    minLength?: Min;
    maxLength?: Max;
}): ArrayBounds<T, {
    minLength: Min;
    maxLength: Max;
}>;
export interface record<S> {
    toJsonSchema(): {
        type: 'object';
        additionalProperties: Returns<S['toJsonSchema' & keyof S]>;
    };
}
export declare function record<T>(additionalProperties: T): {
    type: 'object';
    additionalProperties: T;
};
export interface union<S> {
    toJsonSchema(): {
        anyOf: {
            [I in keyof S]: Returns<S[I]['toJsonSchema' & keyof S[I]]>;
        };
    };
}
export declare function union<T extends readonly unknown[]>(children: T): {
    anyOf: any[];
};
export interface intersect<T> {
    toJsonSchema(): {
        allOf: {
            [I in keyof T]: Returns<T[I]['toJsonSchema' & keyof T[I]]>;
        };
    };
}
export declare function intersect<T extends readonly unknown[]>(children: T): {
    allOf: any[];
};
export interface tuple<S> {
    toJsonSchema(): {
        type: 'array';
        items: {
            [I in keyof S]: Returns<S[I]['toJsonSchema' & keyof S[I]]>;
        };
        additionalItems: false;
        minItems: MinItems<S>;
        maxItems: S['length' & keyof S];
    };
}
export declare function tuple<T extends readonly unknown[]>(children: T): {
    type: "array";
    additionalItems: false;
    items: never;
    minItems: never;
    maxItems: number;
};
export { object_ as object };
interface object_<T, KS extends RequiredKeys<T> = RequiredKeys<T>> {
    toJsonSchema(): {
        type: 'object';
        required: {
            [I in keyof KS]: KS[I] & string;
        };
        properties: {
            [K in keyof T]: Returns<T[K]['toJsonSchema' & keyof T[K]]>;
        };
    };
}
declare function object_<T extends {
    [x: string]: unknown;
}, KS extends RequiredKeys<T>>(children: T): {
    type: 'object';
    required: {
        [I in keyof KS]: KS[I] & string;
    };
    properties: {
        [K in keyof T]: Returns<T[K]['toJsonSchema' & keyof T[K]]>;
    };
};
//# sourceMappingURL=jsonSchema.d.ts.map